<?php

namespace Drupal\multilingual_exclude\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ThemeHandlerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure example settings for this site.
 */
class MultilingualExcludeSettingsForm extends ConfigFormBase {

  /**
   * Config settings.
   *
   * @var string
   */
  const SETTINGS = 'multilingual_exclude.settings';

  /**
   * The theme handler.
   *
   * @var \Drupal\Core\Extension\ThemeHandlerInterface
   */
  protected $themeHandler;

  /**
   * The constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Extension\ThemeHandlerInterface $themeHandler
   *   The theme handler.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    ThemeHandlerInterface $themeHandler
  ) {

    parent::__construct($config_factory);
    $this->themeHandler = $themeHandler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('theme_handler')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'multilingual_exclude_admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      static::SETTINGS,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(
    array $form,
    FormStateInterface $form_state
  ) {

    // Have at least empty array to use.
    $routes = [];

    // Adding the ability to do ajax forms.
    $form['#tree'] = TRUE;

    // Setup the add_more_called flag, so that we know
    // which of the items to load.
    // If we even do one add or remove, we set this flag
    // so that the form will load the values from the
    // form_state, if flag is not set, it will load
    // from the block config.
    if (empty($form_state->get('add_more_called'))) {
      $add_more_called = FALSE;
      $form_state->set('add_more_called', $add_more_called);
    }
    else {
      $add_more_called = $form_state->get('add_more_called');
    }

    // If the add_more_flag is not set, we load from the
    // block config.  If it is set, we load from the form_state.
    if (!$add_more_called) {
      $config = $this->config(static::SETTINGS);
      $routes = $config->get('routes');
      $form_state->set('routes', $routes);
    }
    else {

      $routes = $form_state->get('routes');
    }

    // Get the num_of_rows from the form_state.
    $num_of_rows = $form_state->get('num_of_rows');

    // If the num_of_rows is not set, we first look at
    // the config and see if we have ecs, and if
    // so set num_of_rows to the number of routes.  If we
    // do not have num_of_rows set, default to 1 (this is
    // the first load of the form).
    if (empty($num_of_rows)) {
      if (isset($routes)) {
        $num_of_rows = count($routes);
      }
      else {
        $num_of_rows = 1;
      }
    }

    // Set the num_of_rows to the form_state so that we
    // can use it in the ajax calls.
    $form_state->set('num_of_rows', $num_of_rows);

    // The class to be used for groups.
    $group_class = 'group-order-weight';

    // The field set for the routes groups.
    $form['routes_fieldset'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Routes'),
      '#prefix' => '<div id="routes-wrapper">',
      '#suffix' => '</div>',
    ];

    // Build the table.
    $form['routes_fieldset']['items'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('Route'),
        $this->t('Theme'),
      ],
      '#empty' => $this->t('<span>There are currently no routes set to be excluded.</span>'),
      '#tableselect' => FALSE,
      '#prefix' => '<div class="routes__table">',
      '#suffix' => '</div><p>Leave a route blank to remove it.</p>',
    ];

    // Step through the number of rows and add the config
    // data to the table.
    for ($i = 0; $i < $num_of_rows; $i++) {

      // Reset the settings array.
      $settings = [];

      // The first column of the table, that will house the arrows
      // for rearranging e/c groups.
      $settings['route'] = [
        '#type' => 'textfield',
        '#default_value' => $routes[$i]['route'] ?? NULL,
      ];

      // Container for the group.
      $settings['theme'] = [
        '#type' => 'select',
        '#options' => $this->getAllThemeOptions(),
        '#empty_option' => $this->t('-- Select --'),
        '#default_value' => $routes[$i]['theme'] ?? '',
      ];

      // Add the settings to the items array, which is full row
      // in the table.
      $form['routes_fieldset']['items'][] = $settings;
    }

    // The add expand/collapse group button.
    $form['routes_fieldset']['add_route'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add route'),
      '#submit' => [[$this, 'routeAddOne']],
      '#ajax' => [
        'callback' => [$this, 'routeCallback'],
        'wrapper' => 'routes-wrapper',
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * Add one more e/c section to the form.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function routeAddOne(array &$form, FormStateInterface $form_state) {

    // Set the add_more_called flag, so that we will load
    // the items by form_state, rather than config.
    $form_state->set('add_more_called', TRUE);

    // Get the current values of the form_state.
    $values = $form_state->getValues();

    // Get the items from the form state, which will contain
    // the route settings.
    $items = $values['routes_fieldset']['items'];

    // Define a variable routes.
    $routes = [];

    // Prepare variable ecs if the items exists.
    if (!empty($items)) {

      // Increase by 1 the number of rows and set it in
      // the form_state.
      $form_state->set('num_of_rows', count($items) + 1);
    }
    else {
      $form_state->set('num_of_rows', 1);
    }

    // Set the ecs from the items in the form_state, so that we
    // will load newly added values.
    $form_state->set('routes', $routes);

    // Rebuild form with 1 extra row.
    $form_state->setRebuild();
  }

  /**
   * The ajax call back for route callback.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return mixed
   *   The form element to be overwritten.
   */
  public function routeCallback(array &$form, FormStateInterface $form_state) {
    return $form['routes_fieldset'];
  }

  /**
   * Function to get all the themes into selectable options.
   *
   * @return array
   *   Array of themes in a select options format.
   */
  private function getAllThemeOptions(): array {

    // Get all the enabled themes.
    $themes = $this->themeHandler->listInfo();

    // The array to be used for the options.
    $options = [];

    // Step through each of the options and get the
    // themes into an array that can be used in
    // a select list.
    foreach ($themes as $machine_name => $theme) {
      $options[$machine_name] = $theme->info['name'];
    }

    return $options;
  }

  /**
   * @inheritDoc
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

    // Perform the parent validate form.
    parent::validateForm($form, $form_state);

    // Get the values from the form state.
    $values = $form_state->getValues();

    // Counter to be used for any errors.
    $counter = 0;

    // Step through each of the values and get and ensure
    // that there is a route and theme, if not set the
    // form error.
    foreach ($values['routes_fieldset']['items'] as $route) {

      // If there is a route and no theme, set the form error.
      if ($route['route'] !== '' && $route['theme'] == '') {
        $form_state->setError(
          $form['routes_fieldset']['items'][$counter]['theme'],
          $this->t('You must select a theme for the route.')
        );
      }

      // Increment the counter, so we set the correct error.
      $counter++;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(
    array &$form,
    FormStateInterface $form_state
  ): void {

    // Have at least empty array to return.
    $routes = [];

    // Get the values from the form state.
    $values = $form_state->getValues();

    // Step through each of the values and get the route
    // and theme, and put into the array.
    foreach ($values['routes_fieldset']['items'] as $route) {

      // Only add if we have a route.
      if ($route['route'] !== '') {
        $routes[] = [
          'route' => $route['route'],
          'theme' => $route['theme'],
        ];
      }
    }

    // Save the configuration.
    $this->config(static::SETTINGS)
      ->set('routes', $routes)
      ->save();

    parent::submitForm($form, $form_state);
  }

}
